#include <gnome.h>
#include <glade/glade.h>
#include <libgnomevfs/gnome-vfs.h>
#include <gconf/gconf-client.h>

typedef struct obj {
	GladeXML		*gui;
	GtkWidget		*window;
	GnomeProgram		*program;
	GtkWidget		*width;
	GtkWidget		*height;
} OBJECTS;

OBJECTS obj;

static void show_picture (gchar *picture);
GList *get_pictures_list ();
gchar *str_erase (gchar *str, gint pos, gint len);
gchar *get_path ();
static void set_path (gchar *path);

int
main (int argc, char **argv)
{
	gnome_vfs_init ();
	obj.program = gnome_program_init ("IB",
					"0.1",
					LIBGNOMEUI_MODULE,
					argc, argv,
					GNOME_PROGRAM_STANDARD_PROPERTIES,
					NULL);
	
	obj.gui = glade_xml_new ("./share/gui.glade", NULL, NULL); /* fix path */
	glade_xml_signal_autoconnect (obj.gui);
	
	obj.window = glade_xml_get_widget (obj.gui, "ib");
	/* gchar *icon = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_PIXMAP, "ib.png", NUMLL);
	gtk_window_icon_set_from_file (GTK_WINDOW (window), icon);*/
	
	if (argc != 0) {
		set_path (str_erase (gnome_vfs_make_uri_from_shell_arg (argv[1]), 0, 7));
		
		int width, height;
		gdk_pixbuf_get_file_info (get_path (), &width, &height);
		obj.width = glade_xml_get_widget (obj.gui, "spinbutton1");
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.width), width);
		obj.height = glade_xml_get_widget (obj.gui, "spinbutton2");
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.height), height);
		
		show_picture (get_path ());
	} else {
		return -1;
		/* show usage message */
	}
	
	gtk_main ();

	return 0;
}

/**/
GList *
get_pictures_list ()
{
	GList *images		= NULL;
	gchar *full_path	= NULL;
	GnomeVFSURI *VFS	= gnome_vfs_uri_new (get_path ());
	gchar *uri		= gnome_vfs_uri_extract_dirname (VFS);
	GDir *dir		= NULL;
	const gchar *file;
	gchar *mime;
	
	dir = g_dir_open ((gchar *) uri, 0, NULL);
	while ((file = g_dir_read_name (dir)) != NULL) {
		full_path = g_build_path (G_DIR_SEPARATOR_S, uri, file, NULL);
		mime = gnome_vfs_get_mime_type (full_path);
		if (g_str_equal (str_erase (mime, 5, -1), "image")) {
			images = g_list_append (images, full_path);
		}
		full_path = NULL;
	}
	
	g_free (full_path);
	g_dir_close (dir);
	return images;
}

static void
show_picture (gchar *picture)
{
	if (picture == NULL) {
		return;
	}
	
	obj.window = glade_xml_get_widget (obj.gui, "ib");
	gnome_window_toplevel_set_title (GTK_WINDOW (obj.window), picture, "Image Browser", ".img");
	
	GtkWidget *image = glade_xml_get_widget (obj.gui, "image1");

	gint width = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (obj.width));
	gint height = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (obj.height));
	
	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size (picture, (gint) width, (gint) height, NULL);
	
	if (g_file_test (picture, G_FILE_TEST_EXISTS)) {
		gtk_image_set_from_pixbuf (GTK_IMAGE (image), GDK_PIXBUF (pixbuf));
	} else {
		gtk_image_set_from_stock (GTK_IMAGE (image), GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_DIALOG);
	}
	
	GtkWidget *appbar1 = glade_xml_get_widget (obj.gui, "appbar1");
	gnome_appbar_set_status (GNOME_APPBAR (appbar1), 
		g_strdup_printf ("%s - %d * %d", get_path (), width, height));
}

gchar *
str_erase (gchar *str, gint pos, gint len)
{
	GString *string;
	string = g_string_new (str);
	g_string_erase (string, pos, len);
	return (gchar *) string->str;
}

gchar *
get_path ()
{
	GtkWidget *entry = glade_xml_get_widget (obj.gui, "entry1");
	return (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
}

static void
set_path (gchar *path)
{
	GtkWidget *entry = glade_xml_get_widget (obj.gui, "entry1");
	gtk_entry_set_text (GTK_ENTRY (entry), path);
}

/* callbacks */
void
on_ib_delete_event (GtkWidget *widget, gpointer data)
{
	gnome_vfs_shutdown ();
	gtk_main_quit ();
}

void
on_next_clicked (GtkWidget *widget, gpointer data)
{
	GList *list = NULL;
	GList *images = get_pictures_list ();
	for (list = images; list != NULL; list = list->next) {
		if (g_str_equal (get_path (), (gchar *) list->data)) {
			if (g_list_next (list) == NULL) {
				set_path ((gchar *) g_list_first (list)->data);
				int width, height;
				gdk_pixbuf_get_file_info (get_path (), &width, &height);
				obj.width = glade_xml_get_widget (obj.gui, "spinbutton1");
				gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.width), width);
				obj.height = glade_xml_get_widget (obj.gui, "spinbutton2");
				gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.height), height);
				show_picture (get_path ());
				break;
			} else {
				set_path ((gchar *) g_list_next (list)->data);
				int width, height;
				gdk_pixbuf_get_file_info (get_path (), &width, &height);
				obj.width = glade_xml_get_widget (obj.gui, "spinbutton1");
				gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.width), width);
				obj.height = glade_xml_get_widget (obj.gui, "spinbutton2");
				gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.height), height);
				show_picture (get_path ());
				break;
			}
		}
	}
}

void
on_back_clicked (GtkWidget *widget, gpointer data)
{
	GList *list = NULL;
	GList *images = get_pictures_list ();
	for (list = images; list != NULL; list = list->next) {
		if (g_str_equal (get_path (), (gchar *) list->data)) {
			if (g_list_previous (list) == NULL) {
				set_path ((gchar *) g_list_last (list)->data);
				int width, height;
				gdk_pixbuf_get_file_info (get_path (), &width, &height);
				obj.width = glade_xml_get_widget (obj.gui, "spinbutton1");
				gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.width), width);
				obj.height = glade_xml_get_widget (obj.gui, "spinbutton2");
				gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.height), height);
				show_picture (get_path ());
				break;
			} else {
				set_path ((gchar *) g_list_previous (list)->data);
				int width, height;
				gdk_pixbuf_get_file_info (get_path (), &width, &height);
				obj.width = glade_xml_get_widget (obj.gui, "spinbutton1");
				gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.width), width);
				obj.height = glade_xml_get_widget (obj.gui, "spinbutton2");
				gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.height), height);
				show_picture (get_path ());
				break;
			}
		}
	}
}

void
on_sendmail_clicked (GtkWidget *widget, gpointer data)
{
	system (g_strdup_printf ("evolution mailto:?attachment=%s&", get_path ()));
}

void
on_edit_clicked (GtkWidget *widget, gpointer data)
{
	system (g_strdup_printf ("gimp %s&", get_path ()));
}

void
on_normalsize_clicked (GtkWidget *widget, gpointer data)
{
	int width, height;
	gdk_pixbuf_get_file_info (get_path (), &width, &height);
	obj.width = glade_xml_get_widget (obj.gui, "spinbutton1");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.width), width);
	obj.height = glade_xml_get_widget (obj.gui, "spinbutton2");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.height), height);
	show_picture (get_path ());
}

void
on_zoomin_clicked (GtkWidget *widget, gpointer data)
{
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.height), 
		(gtk_spin_button_get_value (GTK_SPIN_BUTTON (obj.height)) + 200));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.width),  
		(gtk_spin_button_get_value (GTK_SPIN_BUTTON (obj.width)) + 200));
	show_picture (get_path ());
}

void
on_zoomout_clicked (GtkWidget *widget, gpointer data)
{
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.height), 
		(gtk_spin_button_get_value (GTK_SPIN_BUTTON (obj.height)) - 200));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (obj.width),  
		(gtk_spin_button_get_value (GTK_SPIN_BUTTON (obj.width)) - 200));
	show_picture (get_path ());
}
